<?php

namespace Myriad\Installations;

use Illuminate\Contracts\Config\Repository;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;
use Myriad\Installations\Models\Installation;
use Myriad\Installations\Validation\InstallationValidationProviderContract;

/**
 * InstallationManager makes it easy to work with Installations by providing
 * a single place to do what you need to do with them.
 */
class InstallationManager
{

    /**
     * Gives access to the data in the HTTP request.
     *
     * @var Request
     */
    private $request;

    /**
     * Gives access to the config info for Installations.
     *
     * @var Repository
     */
    private $config;

    /**
     * Cache of installations.
     * @var Collection|Installation[]
     */
    private $installations;

    /**
     * Knows how to validate requests for Installations.
     *
     * @var InstallationValidationProviderContract
     */
    private $validator;

    /**
     * InstallationManager constructor.
     *
     * @param  Request  $request  Gives access to the data in the HTTP request.
     * @param  Repository  $config  Gives access to the config info for Installations.
     * @param  InstallationValidationProviderContract  $validator  Knows how to validate requests for Installations.
     */
    public function __construct(Request $request, Repository $config, InstallationValidationProviderContract $validator)
    {
        $this->request = $request;
        $this->config = $config;
        $this->validator = $validator;
        $this->installations = collect();
    }

    /**
     * Get the current installation.
     *
     * @return Installation|null
     */
    public function getInstallation()
    {
        $uuid = $this->getUuidFromRequest();
        if (!$this->installations->has($uuid)) {
            $this->createOrUpdateInstallation($uuid);
        }
        return $this->installations->get($uuid);
    }

    public function setRequest(Request $request)
    {
        $this->request = $request;
    }

    /**
     * Find an Installation by its id.
     *
     * @param  int  $id  The id of the Installation to find.
     * @return Installation|null
     */
    public function findInstallationById($id)
    {
        $installation = $this->installations->keyBy('id')->get($id);
        if (!$installation) {
            $installation = Installation::query()->find($id);
            if ($installation) {
                $this->installations->put($installation->uuid, $installation);
            }
        }
        return $installation;
    }

    /**
     * Find an Installation by its UUID.
     *
     * @param  string  $uuid  Installation UUID.
     * @return Installation|null
     */
    public function findInstallationByUUID($uuid)
    {
        $uuid = $this->validator->normalizeUUID($uuid);
        $installation = $this->installations->get($uuid);
        if (!$installation) {
            $installation = Installation::where('uuid', $uuid)->first();
            if ($installation) {
                $this->installations->put($uuid, $installation);
            }
        }
        return $installation;
    }

    /**
     * Get the UUID from the HTTP request.
     *
     * @return string
     */
    public function getUuidFromRequest()
    {
        $uuid = $this->request->header($this->config->get('installations.request_map.app_installation_id.header'),
          $this->request->query($this->config->get('installations.request_map.app_installation_id.query')));
        return $this->validator->normalizeUUID($uuid);
    }

    /**
     * Get the app name from the HTTP request.
     *
     * @return string
     */
    public function getAppNameFromRequest()
    {
        return $this->request->header($this->config->get('installations.request_map.app_name.header'),
          $this->request->query($this->config->get('installations.request_map.app_name.query')));
    }

    /**
     * Get the app version from the HTTP request.
     *
     * @return string
     */
    public function getAppVersionFromRequest()
    {
        return $this->request->header($this->config->get('installations.request_map.app_version.header'),
          $this->request->query($this->config->get('installations.request_map.app_version.query')));
    }

    /**
     * Get the user agent from the HTTP request.
     *
     * @return array|string
     */
    public function getUserAgentFromRequest()
    {
        return $this->request->server('HTTP_USER_AGENT');
    }

    /**
     * Get the default attributes from the request.
     *
     * @return array
     */
    public function getParamsFromRequest()
    {
        return [
          'uuid' => $this->getUuidFromRequest(),
          'app_name' => $this->getAppNameFromRequest(),
          'app_version' => $this->getAppVersionFromRequest(),
          'user_agent' => $this->getUserAgentFromRequest(),
        ];
    }

    /**
     * Create or update an installation for the given UUID and request.
     *
     * @param  string  $uuid  The Installation's UUID.
     * @param  array  $params  The parameters to provide when creating or updating the Installation.
     * @return Installation the installation
     */
    public function createOrUpdateInstallation($uuid, $params = [])
    {
        $uuid = $this->validator->normalizeUUID($uuid);
        $params['uuid'] = $uuid;

        $installation = $this->findInstallationByUUID($uuid);
        if (!$installation) {
            $installation = new Installation();
            $installation->uuid = $uuid;
        }

        $params = array_merge($this->getParamsFromRequest(), $params);
        $params = Arr::only($params, [
          'app_name', 'app_version', 'platform_name', 'platform_version', 'locale', 'timezone', 'push_type', 'push_id',
          'user_agent',
        ]);

        $installation->fill($params);
        $installation->save();

        $this->installations->put($uuid, $installation);

        return $installation;
    }

    /**
     * Get the validation provider.
     *
     * @return InstallationValidationProviderContract
     */
    public function getValidator()
    {
        return $this->validator;
    }

}
